﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace DataAcquisitionCardDemo.ADC
{
    /// <summary>
    /// 由于队列操作存在线程安全，线程同步等问题，导致数据处理速度不够，
    /// 从而引发了数据不连续的问题，因此使用该类解决快速吞吐数据的问题
    /// </summary>
    class DAQBuffer
    {
        /// <summary>
        /// 开辟20MB的缓存空间
        /// </summary>
        public const int BufferSize = 20971520;

        private byte[] buffer;
        private volatile int startIndex;
        private volatile int endIndex;

        public DAQBuffer()
        {
            buffer = new byte[BufferSize];
            startIndex = 0;
            endIndex = 0;
        }

        public bool Add(byte[] datas)
        {
            //判断空间是否足够
            if (datas.Length + endIndex > BufferSize)
            {
                //空间不足，整理数组
                Refresh();
            }

            //整理完数据后，还是空间不足，加入失败
            if (datas.Length + endIndex > BufferSize)
            {
                return false;
            }

            //空间足够，开始往buffer填数据
            Array.Copy(datas, 0, buffer, endIndex, datas.Length);
            endIndex += datas.Length;

            return true;
        }

        public void Get(ref int len,ref byte[] datas)
        {
            int length = Length;
            //如果要获取的长度大于现有数据长度，则取现有数据长度
            if (len > length)
            {
                len = length;
            }

            //如果要取的数据长度不为0，则获取
            if (len > 0)
            {
                Array.Copy(buffer, startIndex, datas, 0, len);
                startIndex += len;
            }
            else
            {
                len = 0;
            }
        }

        public int Length
        {
            get
            {
                return endIndex - startIndex;
            }
        }

        public void Clear()
        {
            Array.Clear(buffer, 0, BufferSize);
            startIndex = 0;
            endIndex = 0;
        }

        /// <summary>
        /// 当buffer存满时，整理buffer，将前面使用过的空间清除，并将数组从0开始对齐
        /// </summary>
        private void Refresh()
        {
            int length = Length;
            byte[] tmp = new byte[length];
            Array.Copy(buffer, startIndex, tmp, 0, length);
            Array.Clear(buffer,0, BufferSize);
            Array.Copy(tmp, buffer, length);
            startIndex = 0;
            endIndex = length;
        }
    }
}
