﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace DataAcquisitionCardDemo.Model
{
    public class EBS
    {
        //当连续FlatCount点之间的电平差值低于VoltShake，表示平稳状态
        public const int FlatCount = 100;
        //当电压浮动范围小于该值，表示电平稳定
        public const double VoltShake = 0.3;

        private int reSampleInterval = 10;

        public EBS() { }

        public void SetReSampleInterval(int reSampleInterval)
        {
            this.reSampleInterval = reSampleInterval;
        }

        /// <summary>
        /// 判断是否有触发信号，如果有，返回true，并设置IsRaiseEdge
        /// </summary>
        /// <param name="triggerWave">触发信号数据</param>
        /// <param name="triggerLevel">传感器的信号电平</param>
        /// <returns>true表示有触发信号，false表示没有触发信号</returns>
        public EdgeType IsTrigger(ref float[] triggerWave,float triggerLevel,out int triggerIndex)
        {
            //根据重采样间隔来获取数据线性特性：上升、下降
            for(int i = reSampleInterval; i < triggerWave.Length; i += reSampleInterval)
            {
                //判断是否有低电平往高电平跳变
                if(triggerWave[i - reSampleInterval] < triggerLevel && triggerWave[i] >= triggerLevel)
                {
                    triggerIndex = i;
                    return EdgeType.Raise;
                }

                //判断是否有高电平往低电平跳变
                if (triggerWave[i - reSampleInterval] >= triggerLevel && triggerWave[i] < triggerLevel)
                {
                    triggerIndex = i;
                    return EdgeType.Fall;
                }
            }

            //非沿跳变
            triggerIndex = -1;
            return EdgeType.None;
        }

        public EdgeIndex FindRaiseIndex(ref List<float> triggerWave, int startIndex, float triggerLevel)
        {
            EdgeIndex raiseEdge = new EdgeIndex(-1,-1);
            int flatCount = 0;

            //寻找沿跳变的位置
            int triggerIndex = 0;
            for (int i = startIndex; i < triggerWave.Count - 1; i++)
            {
                if (triggerWave[i] < triggerLevel && triggerWave[i+1] >= triggerLevel)
                {
                    triggerIndex = i;
                    break;
                }
            }

            //寻找开始上升的第一个位置
            for (int i = triggerIndex; i > 0; i--)
            {
                //找到了不是连续下降的值
                if (Math.Abs(triggerWave[i] - triggerWave[i - 1]) < VoltShake && triggerWave[i] < triggerLevel)
                {
                    //判断是否持续平稳
                    int count = 0;  //统计连续判断的点数
                    flatCount = 0;
                    for (int j = i-1; j >= 0; j--)
                    {
                        if(Math.Abs(triggerWave[i] - triggerWave[j]) <= VoltShake)
                        {
                            flatCount++;
                        }

                        count++;
                        //这里表示已经判断了足够的点数
                        if(count >= FlatCount)
                        {
                            //表示是平稳状态
                            if (flatCount >= FlatCount)
                            {
                                raiseEdge.Start = i;
                            }
                            else
                            {
                                raiseEdge.Start = -1;
                            }
                            break;
                        }
                    }
                }
                
                if (raiseEdge.Start > -1)
                {
                    break;
                }
            }

            //寻找开始上升的最后一个位置
            for (int i = triggerIndex+1; i < triggerWave.Count-1; i++)
            {
                //找到了不是连续上升的值
                if (Math.Abs(triggerWave[i + 1] - triggerWave[i]) < VoltShake && triggerWave[i] >= VoltShake)
                {
                    //判断是否持续平稳
                    int count = 0;  //统计连续判断的点数
                    flatCount = 0;
                    for (int j = i+1; j < triggerWave.Count; j++)
                    {
                        if (Math.Abs(triggerWave[j] - triggerWave[i]) <= VoltShake)
                        {
                            flatCount++;
                        }

                        count++;
                        //这里表示已经判断了足够的点数
                        if (count >= FlatCount)
                        {
                            //表示是平稳状态
                            if (flatCount >= FlatCount)
                            {
                                raiseEdge.End = i;
                            }
                            else
                            {
                                raiseEdge.End = -1;
                            }
                            break;
                        }
                    }

                    if (raiseEdge.End > -1)
                    {
                        break;
                    }
                }
            }

            return raiseEdge;
        }

        public EdgeIndex FindFallIndex(ref List<float> triggerWave,int startIndex, float triggerLevel)
        {
            EdgeIndex fallEdge = new EdgeIndex(-1,-1);
            int flatCount = 0;

            //寻找沿跳变的位置
            int triggerIndex = 0;
            for (int i = startIndex; i < triggerWave.Count - 1; i++)
            {
                if (triggerWave[i] >= triggerLevel && triggerWave[i + 1] < triggerLevel)
                {
                    triggerIndex = i;
                    break;
                }
            }

            //寻找开始下降的第一个位置
            for (int i = triggerIndex; i > 0; i--)
            {
                //找到了不是连续上升的值
                if (triggerWave[i] - triggerWave[i-1] <= 0)
                {
                    //判断是否持续平稳
                    int count = 0;  //统计连续判断的点数
                    flatCount = 0;
                    for (int j = i - 1; j >= 0; j--)
                    {
                        if (Math.Abs(triggerWave[j] - triggerWave[i]) <= VoltShake)
                        {
                            flatCount++;
                        }

                        count++;
                        //这里表示已经判断了足够的点数
                        if (count >= FlatCount)
                        {
                            //表示是平稳状态
                            if (flatCount >= FlatCount)
                            {
                                fallEdge.Start = i;
                            }
                            else
                            {
                                fallEdge.Start = -1;
                            }
                            break;
                        }
                    }
                    
                    if (fallEdge.Start > -1)
                    {
                        break;
                    }
                }
            }

            //寻找开始下降的最后一个位置
            for (int i = triggerIndex+1; i < triggerWave.Count - 1; i++)
            {
                //找到了不是连续下降的值
                if (triggerWave[i] - triggerWave[i + 1] <= 0)
                {
                    //判断是否持续平稳
                    int count = 0;  //统计连续判断的点数
                    flatCount = 0;
                    for (int j = i + 1; j < triggerWave.Count; j++)
                    {
                        if (Math.Abs(triggerWave[i] - triggerWave[j]) <= VoltShake)
                        {
                            flatCount++;
                        }

                        count++;
                        //这里表示已经判断了足够的点数
                        if (count >= FlatCount)
                        {
                            //表示是平稳状态
                            if (flatCount >= FlatCount)
                            {
                                fallEdge.End = i;
                                //寻找到第一个非下降的点
                                for (int k=i+1; k< triggerWave.Count; k++)
                                {
                                    if(triggerWave[k] - triggerWave[k - 1] >= 0.0005)
                                    {
                                        fallEdge.End = k;
                                        break;
                                    }
                                }
                                
                            }
                            else
                            {
                                fallEdge.End = -1;
                            }
                            break;
                        }
                    }
                    if (fallEdge.End > -1)
                    {
                        break;
                    }
                }
            }

            return fallEdge;
        }

        /// <summary>
        /// 获取脉冲信号宽度，单位us
        /// </summary>
        /// <param name="signalWave"></param>
        /// <param name="signalLevel"></param>
        /// <param name="sampleRate"></param>
        /// <param name="edgeIndex"></param>
        /// <returns></returns>
        public EdgePluse FindPluseWidth(ref List<float> signalWave,float signalLevel,double sampleRate,EdgeIndex edgeIndex)
        {
            EdgePluse edgePluse = new EdgePluse();
            //每一个点间隔时间，单位us
            double step = 1000000.0 / (sampleRate / 4);

            //寻找沿跳变前的第一个脉冲宽度
            for (int i = edgeIndex.Start; i > 0; i--)
            {
                if(signalWave[i]< signalLevel && signalWave[i-1]>= signalLevel)
                {
                    int count = 0;
                    edgePluse.StartIndex.End = i - 1;
                    for (int j = i - 1; j >= 0; j--)
                    {
                        if(signalWave[j] >= signalLevel)
                        {
                            count++;
                        }
                        else
                        {
                            break;
                        }
                    }
                    edgePluse.StartIndex.Start = i - 1 - count;
                    edgePluse.StartPluse = step * count;
                    break;
                }
            }

            //寻找沿跳变后的第一个脉冲宽度
            for (int i = edgeIndex.End; i < signalWave.Count-1; i++)
            {
                if (signalWave[i] < signalLevel && signalWave[i + 1] >= signalLevel)
                {
                    int count = 0;
                    edgePluse.EndIndex.Start = i + 1;
                    for (int j = i + 1; j < signalWave.Count; j++)
                    {
                        if (signalWave[j] >= signalLevel)
                        {
                            count++;
                        }
                        else
                        {
                            break;
                        }
                    }
                    edgePluse.EndIndex.End = i + 1 + count;
                    edgePluse.EndPluse = step * count;
                    break;
                }
            }

            return edgePluse;
        }
    }


    public enum EdgeType
    {
        None = 0,
        Raise = 1,
        Fall = 2,
    }

    public class EdgePluse
    {
        public double StartPluse { get; set; }
        public double EndPluse { get; set; }

        public EdgeIndex StartIndex { get; set; }

        public EdgeIndex EndIndex { get; set; }

        public EdgePluse()
        {
            StartPluse = 0;
            EndPluse = 0;
            StartIndex = new EdgeIndex();
            EndIndex = new EdgeIndex();
        }
    }

    public class EdgeIndex 
    {
        public int Start { get; set; }

        public int End { get; set; }

        public EdgeIndex()
        {
            Start = 0;
            End = 0;
        }

        public EdgeIndex(int startIndex,int endIndex)
        {
            Start = startIndex;
            End = endIndex;
        }
    }

}
